
//###########################################################################
//### header
//###########################################################################

#include <avr/io.h> 

#define OUT 1 //output pin
#define IN	0 //input pin
#define T	0 //tristate
#define P	1 //pullup resistor

//###########################################################################
//### functions
//###########################################################################

void initialisation(void)
{
// Input/Output Ports initialization
// Port A initialization
// alles Ausgangspins
// alle Pins 0


DDRA = ((OUT<<DDA0)|(OUT<<DDA1)|(OUT<<DDA2)|(OUT<<DDA3)|(OUT<<DDA4)|(OUT<<DDA5)|(IN<<DDA6)|(IN<<DDA7));
PORTA= (0<<PA0)|(0<<PA1)|(0<<PA2)|(0<<PA3)|(0<<PA4)|(0<<PA5)|(T<<PA6)|(P<<PA7);


// Port B initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 

PORTB=0x00;
DDRB=0x00;

// Port C initialization
// Func7=Out Func6=Out Func5=Out Func4=Out Func3=Out Func2=Out Func1=Out Func0=Out 
// State7=0 State6=0 State5=0 State4=0 State3=0 State2=0 State1=0 State0=0 
PORTC=0x00;
DDRC=0xFF;

// Port D initialization
// Func7=In Func6=In Func5=Out Func4=Out Func3=In Func2=In Func1=Out Func0=In 
// State7=T State6=T State5=0 State4=0 State3=P State2=P State1=0 State0=T 

DDRD = ((IN<<DDD0)|(OUT<<DDD1)|(IN<<DDD2)|(IN<<DDD3)|(OUT<<DDD4)|(OUT<<DDD5)|(OUT<<DDD6)|(OUT<<DDD7));
PORTD= (0<<PD0)|(0<<PD1)|(0<<PD2)|(0<<PD3)|(0<<PD4)|(1<<PD5)|(1<<PD6)|(0<<PD7);
//PORTD=0x0C;

//DDRD=0x32;

// Timer/Counter 0 initialization
// Clock source: System Clock 
// Clock value: Timer 0 stopped
// Mode: Normal top=FFh, Overflow Interrupt enabled
// OC0 output: Disconnected
TCCR0=0x00;
TCNT0=0x00;
OCR0=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer 1 Stopped
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer 1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer 2 Stopped
// Mode: Normal top=FFh
// OC2 output: Disconnected
ASSR=0x00;
TCCR2=0x04;
TCNT2=0x00;
OCR2=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
// INT2: Off
MCUCR=0x00;
MCUCSR=0x00;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x40; //enable timer2 ovf interrupt

// USART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART Receiver: On
// USART Transmitter: On
// USART Mode: Asynchronous
// USART Baud Rate: 9600
UCSRA=0x00;
UCSRB=0x18;
UCSRC=0x86;
UBRRH=0x00;
UBRRL=0x17;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;


//ADC converter init
ADMUX =0x66; //Vref = AVCC, Channel=ADC6 on PortA.7
SFIOR =0x00; //freerunning mode
ADCSRA=0xFF; //ADC on in freerunning mode with prescaler 128 = 125kHz@16MHz cpu clock


} //initialisation
