
//#############################################################################
//#############################################################################
//###
//###      target: ATmega32
//###  programmer: 
//###         IDE: AVR Studio V4.14 Build 589
//###    compiler: gcc WINAVR 2008-06-10
//###        date: 
//###	 hardware: 
//###
//#############################################################################
//#############################################################################

//### hardware definitions ####################################################

#define LED1 		PD5
#define LED2		PD6
#define LED_PORT	PORTD

#define PIEZO		PD7
#define PIEZO_PORT	PORTD

#define ADC_IN		PA6
#define ADC_PORT	PORTA

#define SDIO_3 		PA1
#define CS 			PA0
#define SCLK		PA3
#define IO_UPDATE 	PA2
#define DATA		PA5
#define RESET		PA4
#define AD9958_PORT PORTA  


//### constants definition ####################################################

#define PI 3.1415

//### header files ############################################################

#include <avr/io.h> 
#include <stdint.h> 
#include <stdio.h>
#include <string.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <iom32.h>

#ifndef EEMEM
#define EEMEM __attribute__ ((section (".eeprom")))
#endif

#include "init.h"
#include "timing.h"
#include "uart.h"
#include "AD9958_board.h"


//### variables and data located in EEPROM memory #############################

uint8_t serial_no_string[66] EEMEM = {"Ser.No.0001-xxxx"}; //contains serial number string of max. 65 characters


//### strings and constants located in flash memory ###########################

const char ret_received[]			PROGMEM = "received:";
const char ret_error[]				PROGMEM = "Error! Received unknown command!\r";
const char ret_welcome[]		    PROGMEM = "TDR-Control ready, waiting for commands...\r";
const char ret_parameter_too_long[] PROGMEM = "Error! Parameter too long! Max 64 characters!\r";

const char cmd_get_ser_no[] 		PROGMEM = "get_ser_no\r";
const char cmd_set_ser_no[]			PROGMEM = "set_ser_no\r";
const char cmd_toggle[]     		PROGMEM = "toggle\r";
const char cmd_set_fout_ch1[] 		PROGMEM = "set_fout_ch1\r";
const char cmd_set_fout_ch0[] 		PROGMEM = "set_fout_ch0\r";
const char cmd_128bit_to_FPGA[]		PROGMEM = "128bit_to_FPGA\r";  //data sent from MCU to FPGA via SPI	in 128bit blocks
const char cmd_beep[]				PROGMEM = "beep\r";
const char cmd_read_ADC[]			PROGMEM = "read_ADC\r";
const char cmd_r[]					PROGMEM = "r\r";         //short form of "read_ADC" for fast transmission



//### global variables ########################################################
char  buffer[101];

volatile int ovf_counter=0;



//#############################################################################
//### ISR routines ############################################################
//#############################################################################

//###################################
//### Timer 0 Overflow interrupt ####
//###################################

ISR(TIMER2_OVF_vect)
{

ovf_counter++;
if(ovf_counter >= 500)
{
 LED_PORT ^=(1<<LED1); //Blink LED zum Testen...
 ovf_counter =0x00;
}

}



//#############################################################################
//### functions ###############################################################
//#############################################################################

void ASCII_to_u32var(char param_string[], uint32_t *u32var)
{  
 signed char str_length = 0;
 signed char cnt=0;
 uint32_t u32_buffer = 0;
 uint32_t u32_factor = 1;
 
 str_length = strlen(param_string);
 for(cnt=str_length-1;cnt>=0;cnt--)  //in loop: convert ASCII string to 32bit variable 
  {
   u32_buffer += (param_string[cnt]-48)*u32_factor;  //decimal 48 = ASCII Offset
   u32_factor*=10;
  } //for
 
  *u32var = u32_buffer;
  
} //ASCII_to_u32var   
   


//#############################################################################
//### main ####################################################################
//#############################################################################

int main(void)
{
//### local variables in main #################################################

 char receive_buffer[140];
 char parameter[70];        //parameter enclosed in brackets in command set
 char Command[70];


 unsigned char i,n;  //used in for loops etc...
 unsigned char detected_bracket = 0; 
 unsigned char parameter_error = 0;
 unsigned char stringlength;

 uint32_t value; //variable used for parameter conversion "string-->u32var"
 uint8_t array[17]; //used for 16 bytes = 128 bit serial data string to FPGA

//#############################################################################
//### system initialisation after startup #####################################
//#############################################################################


//+++ init general stuff, I/O Ports,... +++

 initialisation();  //init registers (System,Interrupts,...)

//+++ init USART +++

 uart_init();  //set baud rate, enable receiver and transmitter,...


 uart_puts_p(ret_welcome); // welcome message after reset indicates ready system



//+++ init AD9958 +++

 reset_AD9958();           // reset DDS Board to default state
 general_AD9958_setup();      // general settings, PLL=20 etc...
 
 
 set_fout_ch0(1000001); // 1MHz + 1 Hz // test
              
 set_fout_ch1(1000000); // 1MHz       //test


//+++global Interrupt enable

sei();

//#############################################################################
//### endless loop ############################################################
//#############################################################################

while (1)
      {

       //+++++++++++++++++++++++
	   //*** command decoder ***
	   //+++++++++++++++++++++++

       //+++ wait for RS-232 string +++

	    parameter_error=0; //reset parameter error 
	    
 	    uart_gets(receive_buffer,sizeof(receive_buffer)); //waiting for string terminated with "\r\n"
	   
	   //+++++++++++++++++++++++++++++++++++
	   //+++ split command and parameter +++
       //+++++++++++++++++++++++++++++++++++
	    
		//+++ clear buffers to 0x00 +++

	    for(i=0;i<70;i++)
        {
        Command[i]=0x00;    //clear string
        parameter[i]=0x00;  //clear string
        } //for


        //+++ copy command to Command-buffer until bracket "(" detected +++
       
	    detected_bracket = 0;  //reset index       
        
		stringlength = strlen(receive_buffer);
        for(i=0;i<stringlength;i++)
        {
          if(receive_buffer[i]!=0x28)  //check string for bracket "("
          {
           Command[i]=receive_buffer[i];
          } //if
          else
          {
           detected_bracket = 1;   //store information about bracket
		   Command[i] = '\r'; //terminate command with carriage return
           break;   //quit loop
          } //else                                    
        } //for


       //+++ if bracket found: copy parameter to parameter-buffer +++

        if(detected_bracket == 1)  //if command contains a parameter
        { 
         n = 0;   //counter
         i+= 1;   //address first character of parameter contained in buffer string and skip "("
         do
         {
          if(receive_buffer[i]!=0x29)  //check for bracket ")"
          {
           parameter[n]=receive_buffer[i]; //copy parameter from buffer to correct string  
          
          } //if
          else
          {
           detected_bracket = 0;  //change loop condition to quit loop  
           parameter[n+1]=0x00;  // terminate ASCII string with "NUL"
          }
          i+=1; //increment counters
          n+=1;                     
          
          if(n>65)  //if parameter string too long
          {
           uart_puts_p(ret_parameter_too_long);
           parameter_error=1; 
           break;   
          } //if
          
         }while(detected_bracket == 1);
        
        } //if 


	   //+++ if no parameter error: compare received command with stored command set +++

   if(parameter_error == 0)
     {
	   if(!strcmp_P(Command,cmd_toggle))
	   {
	     LED_PORT ^= (1<<LED1);  // toggle LED: communication test function 
	   }

	   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	   else if(!strcmp_P(Command,cmd_get_ser_no))
	   {
	    
        for(i=0;i<=100;i++)
		{
		 buffer[i]=0; //erase buffer 
		}		

		for(i=0;i<=65;i++)
		{
		 buffer[i]= eeprom_read_byte(&serial_no_string[i]); //read serial number string from eeprom to SRAM buffer
        
		}//for loop

        strcat(buffer,"\r"); //terminate buffer for transmission
		
        uart_puts_p(ret_received);
		uart_puts_p(cmd_get_ser_no);

		delay_x10ms(10); //time for transmitting data to PC via RS232

		uart_puts(buffer);
	   }

       //++++++++++++++++++++++++++++++++++++++++++++++++++++++++	  ,

	   else if(!strcmp_P(Command,cmd_set_ser_no))
	   {
	    n=strlen(parameter);
		for(i=0;i<65;i++)
		{
		 eeprom_write_byte(&serial_no_string[i],0x00); //reset all EEPROM cells to zero for clearing previous string 
		}
		for(i=0;i<n;i++)
		{
		 eeprom_write_byte(&serial_no_string[i],parameter[i]); //copy received parameter string from SRAM to EEPROM
		}

	    uart_puts_p(ret_received);          //echo received command to computer 
		uart_puts("set_ser_no");
		uart_puts("(");
		uart_puts(parameter);
		uart_puts(")\r");

	   }
      //++++++++++++++++++++++++++++++++++++++++++++++++++++++++	  ,

	   else if(!strcmp_P(Command,cmd_128bit_to_FPGA))
	   {

		n=0; //index for converting byte into number

		for(i=0;i<16;i++) //for all 16 bytes = 128 bit
		{
		 array[i]=(((parameter[n]-48)*100)+((parameter[n+1]-48)*10)+(parameter[n+2]-48)); //convert 8bit ASCII to uint8_t variable
		 
		 n+=4; //address next byte in parameter string

		}//for

		uart_puts_p(ret_received);          //echo received command to computer 
		uart_puts("128_bit_to_FPGA");
		uart_puts("(");
		uart_puts(parameter);
		uart_puts(")\r");

	   }
	   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	   else if(!strcmp_P(Command,cmd_set_fout_ch0))
	   {
	    uart_puts_p(ret_received);          //echo received command to computer 
		uart_puts("set_fout_ch0");
		uart_puts("(");
		uart_puts(parameter);
		uart_puts(")\r");

		ASCII_to_u32var(parameter,&value);  //convert received parameter string to 32bit variable
		set_fout_ch0(value);                //set DDS channel 1 output frequency

	   }

	   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	   else if(!strcmp_P(Command,cmd_set_fout_ch1))
	   {
	    uart_puts_p(ret_received);          //echo received command to computer 
		uart_puts("set_fout_ch1");
		uart_puts("(");
		uart_puts(parameter);
		uart_puts(")\r");

		ASCII_to_u32var(parameter,&value);  //convert received parameter string to 32bit variable
		set_fout_ch1(value);                //set DDS channel 1 output frequency

	   }

       //++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	   else if(!strcmp_P(Command,cmd_beep))
	   {
	    
		uart_puts_p(ret_received);          //echo received command to computer 
		uart_puts("beep\r");

		for(i=0;i<255;i++)
		{
		 PIEZO_PORT ^=(1<<PIEZO); //toggle Piezo driving pin
		 delay_us(500);
		}
			for(i=0;i<255;i++)
		{
		 PIEZO_PORT ^=(1<<PIEZO); //toggle Piezo driving pin
		 delay_us(400);
		}
			for(i=0;i<255;i++)
		{
		 PIEZO_PORT ^=(1<<PIEZO); //toggle Piezo driving pin
		 delay_us(300);
		}

	   }
       //++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	   else if(!strcmp_P(Command,cmd_read_ADC))
	   {
	    
		uart_puts_p(ret_received);          //echo received command to computer 
		uart_puts("read_ADC\r");

		i=ADCH; //read 8 most significant bits

        sprintf(buffer,"adc_result=%d\r",i);
		uart_puts(buffer);

	   }

       //++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	   else if(!strcmp_P(Command,cmd_r))
	   {
		i=ADCH; //read 8 most significant bits
		uart_putc(i); //send 1 byte only --> fast transmission!
	   }

	   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	   else
	   {
	    uart_puts_p(ret_error); //falls kein gltiges Kommando: Fehlermeldung
	   }

     } // if(parameter_error == 0)



    }; // while(1)

return 0; 
} //main

//### END ###

