
//###########################################################################
//### header
//###########################################################################

#include <avr/io.h>    // Registernamen fr AVR
#include <avr/pgmspace.h>

//###########################################################################
//### functions
//###########################################################################


void uart_init(void)
{

#ifndef F_CPU			   // falls nicht schon im Makefile definiert
#define F_CPU 16000000UL    // Systemtakt in Hz - Definition als unsigned long beachten!!
#endif
 
#define BAUD 57600UL        // hier Baudrate einstellen


//### Werte fr Baudratenregister berechnen ###
#define UBRR_VAL ((F_CPU+BAUD*8)/(BAUD*16)-1)   // clever runden
#define BAUD_REAL (F_CPU/(16*(UBRR_VAL+1)))     // Reale Baudrate
#define BAUD_ERROR ((BAUD_REAL*1000)/BAUD)      // Fehler in Promille, 1000 = kein Fehler.
 



UCSRB |= (1<<TXEN);                            // UART TX einschalten
UCSRC |= (1<<URSEL)|(1 << UCSZ1)|(1 << UCSZ0); // Asynchron 8N1 

UCSRB |= ( 1 << RXEN );    //enable UART Receiver

UBRRH = UBRR_VAL >> 8;     //Baudratenregister setzen
UBRRL = UBRR_VAL & 0xFF;

} //uart_init



//###########################################################################

void uart_putc(uint8_t character)
{

    while (!(UCSRA & (1<<UDRE)))  // warten bis USART bereit
    {
    }   

    UDR = character;                     // sende Zeichen

} //send_char

//###########################################################################

void uart_puts(char *s)   //puts jetzt unabhngig vom Controllertyp!!
{

   while(*s) //solange keine Null '\0' am Stringende erkannt wird
   {
    uart_putc(*s);  //einzelnes Zeichen senden
	s++;            //nchstes Zeichen adressieren  
   } //while

} //uart_puts

//###########################################################################

void uart_puts_p(const char *text)
{
 char byte;

 while((byte = pgm_read_byte(text))) //solange der string im flash noch nicht zu Ende ist
	{
	 uart_putc(byte); //Zeichen aus dem String an UART bergeben
	 text++; //nchstes Zeichen adressieren
	} //while

} //uart_puts_p


//###########################################################################

uint8_t uart_getc(void)
{
  while (!(UCSRA & (1<<RXC)))   // warten bis Zeichen verfuegbar
        ;
    return UDR;                   // Zeichen aus UDR an Aufrufer zurueckgeben

} //uart_getc

//###########################################################################


void uart_gets(char* Buffer,uint8_t MaxLen)
{
  uint8_t NextChar;
  uint8_t StringLen = 0;
 
  NextChar = uart_getc();         // Warte auf und empfange das nchste Zeichen
 
                                  // Sammle solange Zeichen, bis:
                                  // * entweder das String Ende Zeichen kam
                                  // * oder das aufnehmende Array voll ist
  while( NextChar != '\n' && StringLen < MaxLen - 1 )
   {
    *Buffer++ = NextChar;
    StringLen++;
    NextChar = uart_getc();
   }
 
                                  // Noch ein '\0' anhngen um einen Standard
                                  // C-String daraus zu machen
  *Buffer = '\0';

} //uart_gets


//###########################################################################
